@@ -1,9 +1,9 @@
 /**************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006 2007 2008 2009 2010 2011 2012 2013  Michael Cornelison
+   Copyright 2006-2014 Michael Cornelison
    source URL:  kornelix.com
-   contact: kornelix2@googlemail.com
+   contact: kornelix@posteo.de
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,12 +20,16 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.5.5
+//     zfuncs.h   version  v.5.8
 
+#include <sys/sysinfo.h> 
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/resource.h>
 #include <sys/file.h>
+#include <sys/utsname.h>
+#include <malloc.h>
+#include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,7 +39,6 @@
 #include <stdarg.h>
 #include <fcntl.h>
 #include <pthread.h>
-#include <errno.h>
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
@@ -57,7 +60,7 @@
 
 #define  wstrerror(err) strerror(WEXITSTATUS(err))                         //  get text status for child process
 
-#define  mutex          pthread_mutex_t                                    //  abbreviations
+#define  mutex_tp       pthread_mutex_t                                    //  abbreviations
 #define  mutex_init     pthread_mutex_init
 #define  mutex_lock     pthread_mutex_lock
 #define  mutex_trylock  pthread_mutex_trylock
@@ -69,13 +72,20 @@
 #define  true   1
 #define  false  0
 
+#define TRACE trace(__FILE__,__FUNCTION__,__LINE__);                       //  trace execution by funcion and line no.
+
 //  system functions ======================================================
 
+void printz(cchar *format, ...);                                           //  printf() with immediate fflush()      v.5.8
 void apppause();                                                           //  output message and wait for user
 void apppause(cchar * format, ... );                                       //  same, works like printf
 void zpopup_message(cchar *format, ...);                                   //  popup message, thread safe
+
 void zappcrash(cchar *format, ...);                                        //  crash with popup message in text window
 void catch_signals();                                                      //  catch signals and do backtrace dump
+void trace(cchar *file, cchar *func, int line);                            //  implements TRACE macro
+void tracedump();                                                          //  dump program trace data
+
 void beroot(int argc = 0, char *argv[] = 0);                               //  restart image as root if password OK
 
 double get_seconds();                                                      //  seconds since 2000.01.01
@@ -100,13 +110,6 @@ int  zget_locked(int &param);                                              //  l
 void zput_locked(int &param, int value);                                   //  put and unlock multi-thread parameter
 int  zadd_locked(int &param, int incr);                                    //  increment multi-thread parameter
 
-void gate_threads_enter();                                                 //  allow only one thread at a time within
-void gate_threads_leave();                                                 //    the time betwen enter() and leave()
-
-char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
-void zfree(void *buff);                                                    //  free() with log option
-void zmalloc_report();                                                     //  print memory allocation report
-
 int shell_quiet(cchar *command, ...);                                      //  format/run shell command, return status
 int shell_ack(cchar *command, ...);                                        //   ""  + popup an error message if error
 int shell_asynch(cchar *command, ...);                                     //  start shell command, return immediately
@@ -120,6 +123,7 @@ int checkinstall(cchar *prog, ...);                                        //  c
 char * fgets_trim(char * buff, int maxcc, FILE *, int bf = 0);             //  fgets + trim trailing \n \r (blanks)
 int samedirk(cchar *file1, cchar *file2);                                  //  returns 1 if files in same directory
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
+int renamez(cchar *file1, cchar *file2);                                   //  rename, also across file systems
 
 //  measure CPU time spent in a function or code block within a function
 
@@ -132,6 +136,7 @@ inline void cpu_profile_enter(int fnum)                                    //  a
 {  cpu_profile_timer = cpu_profile_elapsed;  }
 inline void cpu_profile_exit(int fnum)                                     //  at exit from measured code block
 {  cpu_profile_table[fnum] += cpu_profile_elapsed - cpu_profile_timer; }
+int pagefaultrate();                                                       //  monitor own process hard fault rate
 
 //  string macros and functions ===========================================
 
@@ -155,7 +160,7 @@ int  strTrim2(char *dest, cchar *source);                                  //  r
 int  strTrim2(char *string);                                               //  remove leading and trailing blanks
 int  strCompress(char *dest, cchar *source);                               //  remove all blanks incl. imbedded
 int  strCompress(char *string);                                            //  remove all blanks
-int  strncatv(char *dest, int maxcc, cchar *source, ...);                  //  catenate strings (last = 0L) 
+int  strncatv(char *dest, int maxcc, cchar *source, ...);                  //  catenate strings (last = null) 
 int  strcmpv(cchar *string, ...);                                          //  compare to N strings, return 1-N or 0
 void strToUpper(char *dest, cchar *source);                                //  move and conv. string to upper case
 void strToUpper(char *string);                                             //  conv. string to upper case
@@ -166,8 +171,7 @@ int  repl_Nstrs(cchar *strin, char *strout, ...);                          //  c
 void strncpyx(char *out, cchar *in, int ccin);                             //  conv. string to hex format
 void StripZeros(char *pNum);                                               //  1.230000E+12 --> 1.23E+12
 int  blank_null(cchar *string);                                            //  test for blank/null string
-char * strdupz(cchar *string, int more = 0, cchar *tag = 0);               //  duplicate string in nonvolatile memory
-int  strdupz(cchar *source, char *&dest, int more = 0, cchar *tag = 0);    //  same + dest zmalloc if null or too short
+char * strdupz(cchar *string, int more = 0);                               //  duplicate string and add space at end
 int  clean_escapes(char *string);                                          //  replace \x escapes with real characters
 int  utf8len(cchar *utf8string);                                           //  get graphic cc for UTF8 string
 int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);         //  get graphic substring from UTF8 string
@@ -253,18 +257,29 @@ double drandz();                                                           //  b
 void spline1(int nn, float *dx, float *dy);                                //  define a curve using nn data points
 float spline2(float x);                                                    //  return y-value for given x-value
 
-/**************************************************************************/
+//  FIFO queue for text strings, single or dual-thread access =============                                       v.5.7
 
-//  application initialization and administration
+typedef struct  {
+   int      qcap;                                                          //  queue capacity
+   int      qnewest;                                                       //  newest entry position, circular
+   int      qoldest;                                                       //  oldest entry position, circular
+   int      qdone;                                                         //  flag, last entry added to queue
+   char     **qtext;                                                       //  up to qcap text strings
+}  Qtext;
 
-#define ZTXmaxent 2000                                                     //  max. translation strings
-#define ZTXmaxcc 4000                                                      //  max. cc per string
+void Qtext_open(Qtext *qtext, int cap);                                    //  initialize Qtext queue, empty
+void Qtext_put(Qtext *qtext, cchar *format, ...);                          //  add text string to Qtext queue
+char * Qtext_get(Qtext *qtext);                                            //  remove text string from Qtext queue
+void Qtext_close(Qtext *qtext);                                            //  free() leftover strings if any
+
+//  application initialization and administration =========================
 
 int zinitapp(cchar *appname, ...);                                         //  initz. app directories and files   v.4.1
 cchar * get_zprefix();                                                     //  get /usr or /usr/local  ...        v.4.1
 cchar * get_zuserdir();                                                    //  get /home/user/.appname/
-cchar * get_zdatadir();                                                    //  get install directory
+cchar * get_zdatadir();                                                    //  get data directory
 cchar * get_zdocdir();                                                     //  get document directory 
+cchar * get_zicondir();                                                    //  get icon directory                 v.5.7
 int  locale_filespec(cchar *ftype, cchar *fname, char *filespec);          //  get a locale dependent file        v.5.5
 
 void showz_userguide(cchar *context = 0);                                  //  show user guide in new process
@@ -273,6 +288,17 @@ void showz_textfile(cchar *type, cchar *file);                             //  s
 void showz_html(cchar *url);                                               //  show html via preferred browser
 void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
 
+/**************************************************************************/
+
+//  translation functions
+
+#define ZTXmaxent 2000                                                     //  max. translation strings
+#define ZTXmaxcc 4000                                                      //  max. cc per string
+
+void ZTXinit(cchar *lang);                                                 //  setup for message translation
+cchar * ZTX(cchar *english);                                               //  get translation for English message
+cchar * ZTX_missing(int &ftf);                                             //  get missing translations, one per call
+
 /**************************************************************************
    GTK utility functions
 ***************************************************************************/
@@ -295,12 +321,14 @@ void wprintp(GtkWidget *Win);                                              //  p
 
 //  intercept text window mouse click functions
 
-typedef void clickfunc_t(GtkWidget *widget, int &line, int &pos);          //  function to get clicked text position
+typedef void clickfunc_t(GtkWidget *widget, int line, int pos);            //  function to get clicked text position
 void textwidget_set_clickfunc(GtkWidget *widget, clickfunc_t clickfunc);   //  (wrapped lines are one logical line)
-char * textwidget_get_line(GtkWidget *widget, int line);                   //  get entire line at clicked position
+char * textwidget_get_line(GtkWidget *widget, int line, int hilite);       //  get entire line at clicked position
 char * textwidget_get_word(char *line, int pos, cchar *dlims, char &end);  //  get delimited word at clicked position
 
-/**************************************************************************/
+//  get the screen and mouse for a given widget
+
+int get_mouse_device(GtkWidget *, GdkScreen **screen, GdkDevice **mouse);  //  return pointers to screen and mouse
 
 //  functions to simplify building menus, tool bars, status bars
 
@@ -309,16 +337,16 @@ char * textwidget_get_word(char *line, int pos, cchar *dlims, char &end);  //  g
 
 #define  zdcbmax 100                                                             //  max. combo box drop-down list
 
-typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
+typedef void cbFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
 
 GtkWidget * create_menubar(GtkWidget *vbox);                                     //  create menubar in packing box
-GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, mtFunc func = 0);    //  add menu item to menubar 
+GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, cbFunc func = 0);    //  add menu item to menubar 
 GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *subname,                   //  add submenu item to menu item 
-                             mtFunc func = 0, cchar *mtip = 0);                  //    with opt. function and popup tip
+                             cbFunc func = 0, cchar *mtip = 0);                  //    with opt. function and popup tip
 
 GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24);            //  toolbar in packing box (no vert gtk3)
-GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *name, cchar *ttip,  //  add button with stock (gtk-quit) 
-                                 cchar *icon, mtFunc func);                //    or custom icon (iconfile.png)
+GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *lab, cchar *tip,    //  add button with label, tool-tip, icon
+                                 cchar *icon, cbFunc func);
 
 GtkWidget * create_stbar(GtkWidget *vbox);                                 //  create status bar in packing box
 int stbar_message(GtkWidget *stbar, cchar *message);                       //  display message in status bar
@@ -327,7 +355,7 @@ int stbar_message(GtkWidget *stbar, cchar *message);                       //  d
 
 GtkWidget * create_popmenu();                                              //  create an empty popup menu
 GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname,             //  add menu item to popup menu
-                           mtFunc func, cchar *arg, cchar *mtip = 0);
+                           cbFunc func, cchar *arg, cchar *mtip = 0);
 void popup_menu(GtkWidget *, GtkWidget *popmenu);                          //  pop-up menu at current mouse posn.
 
 /**************************************************************************/
@@ -336,33 +364,38 @@ void popup_menu(GtkWidget *, GtkWidget *popmenu);                          //  p
 //  menus can be added and arranged using the mouse
 
 typedef void gmenuz_cbfunc(cchar *menu);                                   //  caller-supplied callback function
-void gmenuz(GtkWidget *parent, cchar *ufile, gmenuz_cbfunc);               //  show window, handle mouse drag/click
+void gmenuz(GtkWidget *parent, cchar *title, cchar *ufile, gmenuz_cbfunc); //  show window, handle mouse drag/click
 
 /**************************************************************************/
 
 //  create vertical menu/toolbar in vertical packing box                   //  v.5.5
 
-struct vbmenuent {                                                         //  menu data from caller
-   cchar       *name;                                                      //  menu name
+struct vmenuent {                                                          //  menu data from caller
+   cchar       *name;                                                      //  menu name, text
    cchar       *icon;                                                      //  opt. icon file name
    cchar       *desc;                                                      //  description (mouse hover popup)
-   mtFunc      *func;                                                      //  callback func (GtkWidget *, cchar *arg)
+   cbFunc      *func;                                                      //  callback func (GtkWidget *, cchar *arg)
    cchar       *arg;                                                       //  callback arg
-   GdkPixbuf   *pixbuf1;                                                   //  icon pixbuf or null
-   GdkPixbuf   *pixbuf2;                                                   //  darkened pixbuf (clicked)
-   int         ypos, yhh;                                                  //  layout position, extent
+   GdkPixbuf   *pixbuf;                                                    //  icon pixbuf or null
+   PangoLayout *playout1, *playout2;                                       //  normal and bold menu text
+   int         namex, namey;                                               //  menu name position in layout
+   int         iconx, icony;                                               //  menu icon position
+   int         ylo, yhi;                                                   //  menu height limits
+   int         iconww, iconhh;                                             //  icon width and height
 };
 
-struct vbox_menu {
+struct Vmenu {
    GtkWidget   *vbox;                                                      //  parent window
    GtkWidget   *layout;                                                    //  drawing window
+   int         xmax, ymax;                                                 //  layout size                     v.5.8
    int         mcount;                                                     //  menu entry count
-   vbmenuent   menu[100];
+   vmenuent    menu[100];
 };
 
-vbox_menu *vbox_menu_new(GtkWidget *vbox);                                 //  create new menu in parent vbox
-void vbox_menu_add(vbox_menu *vbm, cchar *name, cchar *icon,               //  add menu item with response function
-                         cchar *desc, mtFunc func, cchar *arg);            //  function may be popup_menu() 
+Vmenu *Vmenu_new(GtkWidget *vbox);                                         //  create new menu in parent vbox
+void Vmenu_add(Vmenu *vbm, cchar *name, cchar *icon,                       //  add menu item with response function 
+               int iconww, int iconhh, cchar *desc, 
+               cbFunc func, cchar *arg);                                   //  function may be popup_menu() 
 
 /**************************************************************************/
 
@@ -370,7 +403,7 @@ void vbox_menu_add(vbox_menu *vbm, cchar *name, cchar *icon,               //  a
 //   widget types: dialog, hbox, vbox, hsep, vsep, frame, scrwin, label, entry, edit, radio, 
 //                 check, button, togbutt, spin, combo, comboE, hscale, vscale, colorbutt
 
-#define zdmaxwidgets 100                           //  v.5.3
+#define zdmaxwidgets 300                           //  v.5.8
 #define zdmaxbutts 10
 #define zdsentinel 2138687954
 
@@ -390,7 +423,7 @@ struct zwidget  {
 };
 
 struct zdialog  {
-      int         sentinel;                        //  validity sintenel
+      int         sentinel;                        //  validity sentinel
       void        *eventCB;                        //  widget event user callback function
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
@@ -421,7 +454,7 @@ int zdialog_set_limits(zdialog *zd, cchar *name, double min, double max);  //  s
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
 int zdialog_run(zdialog *zd, zdialog_event = 0, cchar *posn = 0);          //  run dialog, handle events
-void KBstate(int key, int state);                                          //  extern: pass KB events to main app
+void KBstate(GdkEventKey *event, int state);                               //  extern: pass KB events to main app
 
 int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog
 int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status
@@ -456,15 +489,6 @@ int zdialog_inputs(cchar *action);                                         //  l
 int zdialog_save_inputs(zdialog *zd);                                      //  save zdialog input fields when finished
 int zdialog_restore_inputs(zdialog *zd);                                   //  restore zdialog inputs from prior use
 
-/**************************************************************************/
-
-//  translation functions
-
-void ZTXinit(cchar *lang);                                                 //  setup for message translation
-cchar *ZTX(cchar *english);                                                //  get translation for English message
-void ZTX_translation_start(GtkWidget *parent);                             //  start online translation
-int ZTXnewtranslations();                                                  //  new translations were added
-
 //  write text to popup window, shell command to popup window
 
 GtkWidget * write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
@@ -475,17 +499,19 @@ int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent
 void zmessageACK(GtkWidget *parent, cchar *title, cchar *format, ... );             //  display message, wait for OK
 void zmessLogACK(GtkWidget *parent, cchar *format, ... );                           //  same, with log to STDOUT
 int zmessageYN(GtkWidget *parent, cchar *format, ... );                             //  display message, wait for YES/NO
-void zmessage_help(GtkWidget *parent, cchar *topic, cchar *format, ...);            //  message box, user guide topic
 zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *format, ...);           //  show message, timeout or cancel
 char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);             //  get short text input from user
 int zdialog_choose(cchar *title, GtkWidget *parent, cchar *format, ...);            //  show message, return choice 
-void poptext_window(cchar *text, GtkWindow *pwin, int mx, int my, int secs = 0);    //  show popup text at window posn
-void poptext_mouse(GdkDevice *mouse, cchar *text, int dx, int dy, int secs = 0);    //  show popup text at mouse posn
+void poptext_mouse(cchar *text, GdkDevice *, int dx, int dy, float s1, float s2);   //  show popup text at mouse posn
+void poptext_window(cchar *text, GtkWindow *, int mx, int my, float s1, float s2);  //  show popup text at window posn
+int  poptext_killnow();                                                             //  kill current popup window
+int  popup_image(cchar *imagefile, GtkWindow *parent, int size);                    //  show image in a small popup window
+int  move_pointer(GtkWidget *, int px, int py);                                     //  move the mouse pointer to px, py
 
 //  file chooser dialogs for one file or multiple files
 
-char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
-char ** zgetfileN(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
+char  * zgetfile(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
+char ** zgetfiles(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 
 //  print an image file, choosing printer, paper, orientation, margins, and scale
 
@@ -500,7 +526,6 @@ void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  c
 
 GdkPixbuf * get_thumbnail(char *fpath, int size);                          //  get sized thumbnail for image file
 GdkCursor * zmakecursor(cchar *iconfile);                                  //  make a cursor from an image file
-int gdk_window_move_pointer(GdkWindow *, int px, int py);                  //  move the mouse pointer to px, py
 GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, float deg, int alfa = 0);       //  rotate pixbuf through any angle
 
 /**************************************************************************/
@@ -599,7 +624,7 @@ class  Queue
 {
    char        wmi[8];
    Vxstring  * vd;                                                         //  vector of xstrings
-   mutex       qmutex;                                                     //  for multi-thread access
+   mutex_tp    qmutex;                                                     //  for multi-thread access
    int         qcap;                                                       //  queue capacity
    int         qcount;                                                     //  curr. queue count
    int         ent1;                                                       //  first entry pointer
